(function() {

dojo.provide("jazz.jsp._ServerTemplated");

dojo.require("dijit._Templated");

var _Templated = dijit._Templated;

var buildDijitRendering = _Templated.prototype.buildRendering;

dojo.extend(dijit._Templated, {
	stopParser : true,
	serverTemplated : false,
	
	buildDijitRendering: buildDijitRendering,
	
	buildRendering: function () {
		if (this.serverTemplated)
			this.buildServerRendering();
		else
			this.buildDijitRendering();
	},
	
	buildServerRendering: function() {
		var node = this.srcNodeRef;
		/* _Contained.getParent() is broken without this because
		 * even though the parent has registered itself by its "id" attribute,
		 * the lookup requires the "widgetId" attribute to be set.
		 */
		if (this.id)
			node.setAttribute("widgetId", this.id);

		this.domNode = node;
		delete this.srcNodeRef;

		this._newAttachTemplateNodes(node);

		if(this.widgetsInTemplate) {
			// Store widgets that we need to start at a later point in time
			var cw = (this._startupWidgets = dojo.parser.parse(node, {
				noStart: !this._earlyTemplatedStartup,
				template: false,
				inherited: {dir: this.dir, lang: this.lang},
				propsThis: this,	// so data-dojo-props of widgets in the template can reference "this" to refer to me
				scope: "dojo"	// even in multi-version mode templates use dojoType/data-dojo-type
			}));

			this._supportingWidgets = dijit.findWidgets(node);

			this._newAttachTemplateNodes(cw, function(n,p) {
				return n[p];
			});
		}
	},
	
	_newAttachTemplateNodes: function(rootNode, getAttrFunc) {
		getAttrFunc = getAttrFunc || function(n,p){ return n.getAttribute(p); };
		
		var accumulator = [];
		var nodes = (dojo.isArray(rootNode) ? rootNode : rootNode.children);
		for (var i = 0; i < nodes.length; i++) {
			var node = nodes[i];
			accumulator.push(node);
		}
		
		while (0 < accumulator.length) {
			// Get the next node for processing
			var currentNode = accumulator.pop();
	
			// Process it's attach points
			var attachPoint = getAttrFunc(currentNode,"dojoAttachPoint") || getAttrFunc(currentNode,"data-dojo-attach-point");
			
			if (attachPoint) {
				var point, points = attachPoint.split(/\s*,\s*/);
				while((point = points.shift())) {
					if (dojo.isArray(this[point])) {
						this[point].push(currentNode);
					} else {
						this[point] = currentNode;
					}
					this._attachPoints.push(point);
				}
			}
			
			// Process dojoAttachEvent
			var attachEvent = getAttrFunc(currentNode, "dojoAttachEvent") || getAttrFunc(currentNode,"data-dojo-attach-event");
			if (attachEvent) {
				// NOTE: we want to support attributes that have the form
				// "domNode: nativeElement; ..."
				var widget, event, events = attachEvent.split(/\s*,\s*/), trim = dojo.trim;
				while((event = events.shift())){
					if (event) {
						widget = this;
						var thisFunc = null;
						if (event.indexOf(":") != -1) {
							// oh, if only JS had tuple assignment
							var tokens = event.split(":");
							event = trim(tokens[0]);
							thisFunc = trim(tokens[1]);
							if (tokens[2]) {
								widget = dijit.byId(tokens[2]);
							}
						} else {
							event = trim(event);
						}
						if (!thisFunc) {
							thisFunc = event;
						}
						widget._attachEvents.push(widget.connect(currentNode,event,thisFunc));
					}
				}
			}
			
			// waiRole, waiState
			// TODO: remove this in 2.0, templates are now using role=... and aria-XXX=... attributes directly
			var role = getAttrFunc(currentNode, "waiRole");
			if (role) {
				dijit.setWaiRole(currentNode,role);
			}
			var values = getAttrFunc(currentNode,"waiState");
			if (values){
				dojo.forEach(values.split(/\s*,\s*/), function(stateValue){
					if (stateValue.indexOf('-') != -1){
						var pair = stateValue.split('-');
						dijit.setWaiState(currentNode,pair[0],pair[1]);
					}
				});
			}
			
			// Add interesting children to the node list
			if (this.widgetsInTemplate && (getAttrFunc(currentNode, "dojoType") || getAttrFunc(currentNode, "data-dojo-type"))) {
				continue;
			}
			if (currentNode.children) {
				for (var i = 0; i < currentNode.children.length; i++) {
	            	var node = currentNode.children[i];
	            	accumulator.push(node);
	        	}
        	}
		}
	}
});

})();
