/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {
	
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.ServerRenameStatus");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.ProgressBar");
dojo.require("jazz.client.xhr");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.StyledBox");
dojo.require("net.jazz.ajax.contextRoot");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin", "ServerRenameStatusMessages");

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;
var messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin", "ServerRenameStatusMessages");
var TrimStyles = jazz.ui.StyledBox.TrimStyles;
var REFRESH_DELAY = 3000;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.ServerRenameStatus", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/templates/ServerRenameStatus.html"),
	widgetsInTemplate: true,
	innerBoxTrim:  TrimStyles.WIZARD,
	
	constructor: function() {
		this.messages = messages;
		document.title = this.messages.boxTitle;
	},
	
	postCreate: function() {
		this.getUserRoles();
	},
	
	getUserRoles: function () {
		var srh= new ServiceResponseHandler(this, "authRequestPassed", "authRequestFailed");		
		AdminClient.getUserRoles(srh);
	},
	
	authRequestPassed: function(response) {
		if(response && response.userId) {
			dojo.setObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_USERID", response.userId);
		}
		var roles = response.roles;
		for (var i=0; i < roles.length; i++) {
			if (roles[i] === "JazzAdmins") {
				this.authorized = true;
				break;
			}
		}
		this.startRename();
	},
	
	authRequestFailed: function() {
		this.startRename();
	},
	
	startRename: function() {
		if (!this.authorized) {
			var notAuth = new com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView({ customNotAuthorizedMessage: this.messages.renameInProgress });
			dojo.place(notAuth.domNode, this.notAuthView, "only");
			return;
		}
		this.mainContent.style.display = "block";
		var _queryParams = dojo.queryToObject(window.location.search.slice(1));
		if (_queryParams.internal && _queryParams.internal=="true") {
			this._internalResetStatusButton.style.display="block";
			this.connect(this._internalResetStatusButton, "onclick", this._resetRenameStatus);
		}
		else {
			this.header = new jazz.ui.MessageBox();
			this.headerArea.appendChild(this.header.domNode);
			this._scheduleRefresh(0);
			this.exitLink.href = net.jazz.ajax.contextRoot() + "/admin";
			this.header.showMessage({
				type: "INFO",
				summary: this.messages.renameInProgress
			});
		}
	},
	
	_resetRenameStatus : function() {
		var confirmation=confirm(this.messages.renameStatusResetWarning);
		if (confirmation==true) {
			var srh = new ServiceResponseHandler(this, "_handleReset", "_errorResetingStatus");
       		ServerStatusClient.postServerRenameOverallStatus(srh);
		}		
	},
	
	_handleReset: function() {
		alert(this.messages.renameStatusReset);
	},
	
	_errorResetingStatus : function() {
		alert(this.messages.renameStatusResetError);
	},
	
	_scheduleRefresh: function(time) {
		this._refreshTimeout = setTimeout(dojo.hitch(this, this.refresh), time);
	},
	
	refresh: function() {
		var srh = new ServiceResponseHandler(this, "handleProgressInfo", "errorFetchingProgress");
        ServerStatusClient.getServerRenameOverallStatus(srh);
	},
	
	handleProgressInfo: function(data) {
		if(this._refreshTimeout) {
			clearTimeout(this._refreshTimeout);
			delete this._refreshTimeout;	
		}
		
		if(!data) {
			//No Rename in progress
			this.header.showMessage({
				type: "INFO",
				summary: this.messages.renameNotStarted
			});
			dojo.removeClass(this.exitLink, "noDisplay");
			return;	
		}
		
		//Delete the DOM content
		dojo.empty(this.progressContainer);
		
		//Track overall rename status
		this.someAppsRunning = false;
		this.someAppsHaveErrors = false;
		
		this._progressBars = [];
		this._messageBoxes = [];
		dojo.forEach(data, function(app) {
			var isRunning = app.state === "IN_PROGRESS";
			if(isRunning) {
				this.someAppsRunning = true;
			}
			var div = dojo.create("div", { className: "application" }, this.progressContainer);
			dojo.create("h3", { className: "displayName", innerHTML: app.name }, div);
			
			var percent = app.percentComplete || 0;
			this.progressBar = new dijit.ProgressBar({ progress: percent + "%" });
			dojo.addClass(this.progressBar.domNode, "progressBar");
			div.appendChild(this.progressBar.domNode);
			this.progressBar.startup();
			this._progressBars.push(this.progressBar);
			
			div._errDiv = dojo.create("div", { className: "applicationError noDisplay" }, div);
			div._errorMsgBox = new jazz.ui.MessageBox();
			this._messageBoxes.push(div._errorMsgBox);
			div._errDiv.appendChild(div._errorMsgBox.domNode);
			
			//Show the inline message for Errors/Warnings even if the rename is still in progress
			if(app.resultMessage && (app.result == "ERROR" || app.result == "WARNING")) {
				this._showInlineError(div, app);
			}
			
			if(!isRunning) {
				dojo.addClass(div, app.result);
				if(app.result == "ERROR") {
					this.someAppsHaveErrors = true;
				}
			}

		}, this);
		
		if(this.someAppsRunning) {
			this._scheduleRefresh(REFRESH_DELAY);
		} else {
			//Rename complete
			if(this.someAppsHaveErrors) {
				this.header.showMessage({
					type: "ERROR",
					summary: this.messages.renameFailure
				});	
			} else {
				this.header.showMessage({
					type: "OK",
					summary: this.messages.renameSuccess
				});
			}
			dojo.removeClass(this.exitLink, "noDisplay");
		}
	},
	
	_showInlineError: function(node, app) {
		dojo.removeClass(node._errDiv, "noDisplay");
		node._errorMsgBox.showMessage({
			type: app.result,
			summary: app.resultMessage
		});
	},
	
	errorFetchingProgress: function(errObject) {
		this.header.showMessage({
			errorObject: errObject,
			summary: this.messages.errorFetchingStatus
		});	
	},
	
	uninitialize: function () {
		this.header.destroyRecursive();
		
		//kill the errorDiv message boxes
		if(this._messageBoxes) {
			dojo.forEach(this._messageBoxes, function(mb) {
				mb.destroyRecursive();
			});	
		}
		
		//kill the progress bar widgets
		if(this._progressBars) {
			dojo.forEach(this._progressBars, function(pb) {
				pb.destroyRecursive();
			});	
		}
		this.inherited(arguments);
    }

});

})();