/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function(){
    dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.LicenseTokenServiceProvidersView");
    
    dojo.require("dijit._Widget");
    dojo.require("dijit._Templated");
    dojo.require("net.jazz.ajax.contextRoot");
    dojo.require("net.jazz.ajax.core.internal.util");
    dojo.require("dojox.xml.parser");
    dojo.require("jazz.core.loader");
    dojo.require("com.ibm.team.repository.web.client.internal.licensing");
    dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
    
    dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.PropertiesWizard");
    dojo.require("dojo.i18n");
    dojo.require("jazz.ui.Dialog");
    dojo.require("com.ibm.team.repository.web.ui.internal.common.StyledTable");
    dojo.require("com.ibm.team.repository.web.ui.internal.common.Toolbar");
        
    dojo.require("jazz.ui.MessageBox");
    dojo.require("com.ibm.team.repository.web.client.internal.HttpHeaders");
    dojo.require("jazz.client.xhr");
    
    var MessageBox = jazz.ui.MessageBox;
    var loader = jazz.core.loader;
    var RegistryFactory = jazz.core.RegistryFactory;
    var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;   
    var ExtensionsHelper = com.ibm.team.repository.web.ui.internal.admin.server.ExtensionsHelper;
    var clearNodes = dojox.xml.parser.removeChildren;
    var NO_DISPLAY = "noDisplay";
    var StyledTable = com.ibm.team.repository.web.ui.internal.common.StyledTable;
    var PropertiesWizard = com.ibm.team.repository.web.ui.internal.admin.server.PropertiesWizard;
    var ToolBar = com.ibm.team.repository.web.ui.internal.common.Toolbar;   
    var SRH = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var LICENSE_TOKEN_REST_SVC = "com.ibm.team.repository.service.internal.license.token.ILicenseTokenRestService";
	var PROVIDER_CONFIGS_URL = "com.ibm.team.repository.service.internal.license.token.ILicenseTokenRestService/provider-configs";
	var PROVIDER_STATUS_URL = "com.ibm.team.repository.service.internal.license.token.ILicenseTokenRestService/provider-status";
	
    dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.LicenseTokenServiceProvidersView", [dijit._Widget, dijit._Templated], {
        templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/LicenseTokenServiceProvidersView.html"),
        _enableEditToggle: true,
		 _actionToolbars: [],
        LICENSE_SERVICE_IMPL: "com.ibm.team.repository.service.internal.license.token.LicenseTokenService",
                        
        LICENSE_TOKEN_SERVICE_PROVIDERS_PROP: "tokenProviderService",
        constructor: function(args){
            this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "LicenseEditorMessages");
            this.propertyMessages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ConfigurePageMessages");
        },
        
        postCreate: function(){        
			this.servicePrefix = net.jazz.ajax.contextRoot() + "/service/";
        	this.headers = com.ibm.team.repository.web.client.internal.HttpHeaders._getHeaders();
            this.refresh();
            this.providerTable = new StyledTable({
            
                columns: [{
                    label: "",
                    width: "2%",
                    sortable: false
                }, {
                    label: this.messages.tokenProviderName,
                    width: "30%",
                    sortable: false
                }, {
                    label: this.messages.tokenProviderDesc,
                    sortable: false
                }, {
                    label: "",
                    width: "2%",
                    sortable: false
                }, {
                    label: this.messages.status,
                    width: "10%",
                    sortable: false
                }, {
                    label: this.messages.actions,
					className: "actions",
                    sortable: false
                }],
                sortBy: 1,
                sortByAscending: false,
                handleSort: function(){
                    // do sort
                },
                highlight: true
            });
            
            dojo.addClass(this.providerTable, "actions-table");
            this.providerTableArea.appendChild(this.providerTable.domNode);
            
            dojo.addClass(this.providerTable.tbody , "com-ibm-team-repository-license-tbody");
            
            this.header = new MessageBox();
            this.headerArea.appendChild(this.header.domNode);
        },
        
        
        
        refresh: function(){
            this._fetchProviderConfigs();
        },
        
        // Used to retrieve the config from the floating license service and initialize the 
        // selection of the provider table.  Should only be called on create or revert.        
        _initializeTableSelection: function(configuredProvider){
            deselectAll(this.providerTable);            
            selectProvider(configuredProvider ? configuredProvider : "", this.providerTable);            
        },
                        
        _saveChanges: function(){
        
            var changeArgs = {
                updates: dojo.toJson(this._getChange())
            };
            var srh = new SRH(this, "_submitConfigCallback", "_submitConfigError");
            ServerStatusClient.postConfigurationUpdates(srh, changeArgs);
            
        },
        
        _submitConfigCallback: function(){
        
        },
        
        _submitConfigError: function(errorObj){
            this.displayServiceCallbackErrorMessage(errorObj);
        },
        
        _displayTokenServiceProviders: function(response){
        	if (response)
        	{
            	var self = this;
            	clearNodes(this.providerTable.tbody);
            	// Check to see if we have an error with the service configuration
            	if (response.errorMessage) 
                	this._displayErrorMessage(response.errorMessage);
            
            	var providers = response.licenseTokenProviders;
            	if (providers)
            	{
            		if (providers.length == 0) {
                		this._createNoExtensionsRow();                
            		}
            		else {
                		// Add a none row				 	
                		var noneRow = this._createRow({
                    		name: this.messages.noDefault,
                    		id: ""
                		});
                		noneRow.colSpan = 4;
                		this.providerTable.tbody.appendChild(noneRow);
                		dojo.forEach(providers, dojo.hitch(this, function(config){                			
                    		this.providerTable.tbody.appendChild(this._createRow(config));
                		}));
                
            		}
            	}
            	this._initializeTableSelection(response.configuredProviderId);
            	
            }
        },
                       
        /**
         * finds the declared widget from the web extension point
         * @param {Object} implClass
         */
        _findWidget: function(providerId){
            var extensions = ExtensionsHelper.getLicenseTokenProviders();
            for (var idx = 0; idx < extensions.length; idx++) {
                var ext = extensions[idx];
                if (ext.getId() == providerId) 
                    return ext.getWidget();
            }
            
            return null;
        },
        
        _createNoExtensionsRow: function(){
            var tr = document.createElement("tr");
            var td = document.createElement("td");
            td.className = "noRowsYet";
            td.colSpan = 6;
            td.innerHTML = this.messages.noTokenProviders;
            tr.appendChild(td);
            this.providerTable.tbody.appendChild(tr);
        },
        
        _createRowFromArray: function(data){
            var tr = document.createElement("tr");
            for (var i = 0; i < data.length; i++) {
                var td = document.createElement("td");
                td.appendChild(data[i]);
                tr.appendChild(td);
            }
            return tr;
        },
        
        _fetchStatus: function(providerId , statusIcon , statusMsg){
        	if (providerId)
        	{
        		var messages = this.messages;
        		statusMsg.innerHTML = this.messages.pending;
        		var fetched = {        			        			
        			success: function(response){
        				statusMsg.innerHTML = ""; 
        				if (response && response.status)
        				{       				
        					var status = response.status;            			
                			var claz = getStatusClass(status.code);
                			if (claz) 
                    			dojo.toggleClass(statusIcon, claz.icon, true);
                			if (status.message && status.message != "OK") {
                    			statusMsg.innerHTML = status.message;
                    			if (claz) 
                        			dojo.toggleClass(statusMsg, claz.color, true);
                			}                       		
                		}         		
        			},
        			failure: function(errorObj){
        				var statusClass = PROVIDER_STATUS.ERROR;             		             		
                    	dojo.toggleClass(statusIcon, statusClass.icon, true);                    
            			statusMsg.innerHTML = dojo.string.substitute(messages.errorFetchingStatus, [errorObj.status]);
            			dojo.toggleClass(statusMsg, statusClass.color, true);            
        			}
        		};
        
        		jazz.client.xhrGet({
					url: this.servicePrefix + PROVIDER_STATUS_URL + "?providerId=" + providerId,
					handleAs: "json",
					headers: this.headers,
					load: dojo.hitch(fetched , "success"),
					error: dojo.hitch(fetched , "failure")
				});
			}				       
        },
        
        _createRow: function(config){
            var name = config.name;
            var desc = config.description;
            var id = config.id;
            var implClass = config.implClass;
            var status = config.status;
            var hasProps = config.hasConfig;
            var data = [];
			
			//IE7 cannot create radio buttons the correct way in javascript.
			if(dojo.isIE == 7) {
				var providerCheckbox = document.createElement("<input type=\"radio\" name=\"providerGroup\" />");
			} else {
				var providerCheckbox = dojo.create("input", { type: "radio", name: "providerGroup" });
			}
			
            providerCheckbox.setAttribute("value", implClass);
            providerCheckbox.setAttribute("providerId", id);
			data.push(providerCheckbox);
            this.connect(providerCheckbox, "onclick", "_onSelectProvider");
            // Name            
            var label = document.createElement("label");
            data.push(label);
            label.innerHTML = name;
            
            // Description            
            label = document.createElement("label");
            data.push(label);
            label.appendChild(document.createTextNode(desc || " ")); //IE7
            
            // Status Icon             
            var icon = document.createElement("div");
			icon.appendChild(document.createTextNode(" ")); //IE7
            data.push(icon);
            
            // Status Message			
            var statusMsg = document.createElement("span");
			statusMsg.appendChild(document.createTextNode(" ")); //IE7			
            data.push(statusMsg);
                                    
            // Actions
            var toolBar = new ToolBar();
			this._actionToolbars.push(toolBar);
            if (id) {
            	this._fetchStatus(id , icon , statusMsg);
                var widget = this._findWidget(id);
                if (widget || hasProps) {
                    var editAction = {};
                    editAction.imgSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/images/edit-ellipses.gif");
                    editAction.label = "Edit";
                    editAction.callbackData = {
                        providerId: id,
                        implClass: implClass,
                        svcName: name,
                        widget: widget
                    };
                    var callback = dojo.hitch(this, function(e){
                        this._editProviderConfig(e.currentTarget.callbackData);
                    });
                    
                    toolBar.addButton(this, callback, editAction);
                }
				data.push(toolBar.domNode);
            } else {
				data.push(document.createTextNode(" ")); //IE7
			}

            return this._createRowFromArray(data);
        },
        
        _onSelectProvider: function(e){
            this._saveChanges();
        },
        
        // The properties link on a provider has been selected
        // this shows the configuration widget
        _editProviderConfig: function(config){
            var providerId = config.providerId;
            var implClass = config.implClass;
            var serviceName = config.svcName;
            var widget = config.widget;
            var providerWidget = null;
            
            var saveCallback = dojo.hitch(this, "onProviderConfigChange");
            if (!widget) {
                var parms = {
                    implClass: implClass,
                    serviceName: serviceName,
                    propConfigMessages: {
                        headerTitle: dojo.string.substitute(this.messages.editTokenProviderPropertiesTitle, [serviceName]),
                        dialogTitle: this.messages.editTokenProviderPropertiesDialogTitle
                    },
                    configSaveCallback: saveCallback
                }
                this.providerWidget = new PropertiesWizard(parms);
                
            }
            else {
                var parms = {
                    configSaveCallback: saveCallback
                }
                this.providerWidget = this._createConfigWidget(parms, widget);
                
            }
        },
        
        /**
         * Called when a provider has changed its configuration
         */
        onProviderConfigChange: function(){
            this.refresh();
        },
        
        _createConfigWidget: function(parms, widget){
            // Load widget JS, HTML & CSS		
            loader.load_sync(widget);
            // Instantiate widget
            
            var widgetConstructor = eval(widget);
            var configWidget = new widgetConstructor(parms);
            
            return configWidget;
        },
        
        _getChange: function(){
        
            return [{
                name: this.LICENSE_TOKEN_SERVICE_PROVIDERS_PROP,
                value: getSelectedProviders(this.providerTable),
                serviceImplementationName: this.LICENSE_SERVICE_IMPL
            }];
        },
        
        showStatus: function(message){
            this.header.showMessage(message);
			// Show in the parent as well
            if (this.parentPointer && this.parentPointer.header) {
                this.parentPointer.header.showMessage(message);
            }
        },
        
        clearMessages: function(){
            this.header.hideMessage();
        },
        
        _displayErrorMessage: function(message, errorObj){        
            var args;
            if (errorObj) 
                this.showStatus(errorObj);				
            else {
				this._displayMessage(message , jazz.ui.MessageBox.STATUS_TYPES.ERROR)                
            }
                       
        },
        
        _displaySuccessMessage: function(message){
            this._displayMessage(message, jazz.ui.MessageBox.STATUS_TYPES.OK);
        },
        
        displayServiceCallbackErrorMessage: function(errorObj){
            this.showStatus(errorObj);
        },
        
        // do not call directly
        _displayMessage: function(message, messageType){
            if (this.header) {
                var status = {
                    type: messageType,
                    summary: message
                };
                this.showStatus(status)
            }
        },
        
        _fetchProviderConfigs: function(){   
        	jazz.client.xhrGet({
				url: this.servicePrefix + PROVIDER_CONFIGS_URL,
				handleAs: "json",
				headers: this.headers,
				load: dojo.hitch(this , "_displayTokenServiceProviders"),
				error: dojo.hitch(this , "displayServiceCallbackErrorMessage")
			});	                     	                                    
        },
		
		_destroyActionToolbars: function() {
			for (var i = 0; i < this._actionToolbars.length; i++) {
				this._actionToolbars[i].destroyRecursive();
			}
			this._actionToolbars = [];
		},
		
		uninitialize: function () {
			if(this.providerTable) {
				this.providerTable.destroyRecursive();
			}
			this.header.destroyRecursive();
			this._destroyActionToolbars();
			this.inherited(arguments);
	    }
		
    });
    
    function selectProvider(providerId, providerTable){
        var inputs = providerTable.tbody.getElementsByTagName("input");
        var value = "";
        for (var i = 0; i < inputs.length; i++) {
            if (inputs[i].getAttribute("providerId") == providerId) {
                inputs[i].checked = true;
            }
        }
    }
    
    function deselectAll(providerTable){
        var inputs = providerTable.tbody.getElementsByTagName("input");
        var value = "";
        for (var i = 0; i < inputs.length; i++) {
            inputs[i].checked = false;
        }
    }
    
    function enableProviders(enable, providerTable){
        var inputs = providerTable.tbody.getElementsByTagName("input");
        for (var i = 0; i < inputs.length; i++) {
            inputs[i].disabled = !enable;
        }
    }
    
    function getSelectedProviders(providerTable){
        var inputs = providerTable.tbody.getElementsByTagName("input");
        var value = "";
        for (var i = 0; i < inputs.length; i++) {
            if (inputs[i].checked) {
                value += inputs[i].getAttribute("providerId") + ",";
            }
        }
        if (value.length > 0) 
            value = value.substring(0, value.length - 1);
        return value;
    }
    
    function getSelectedProvider(providerTable){
        var inputs = providerTable.tbody.getElementsByTagName("input");
        var value = null;
        for (var i = 0; i < inputs.length; i++) {
            if (inputs[i].checked) {
                value = inputs[i].getAttribute("providerId");
                break;
            }
        }
        return value;
    }
	
    // Maps to eclipse core runtimes IStatus codes.
    function getStatusClass(code){
        if (code == 0) 
            return PROVIDER_STATUS.OK;
        else 
            if (code == 2) 
                return PROVIDER_STATUS.WARNING;
            else 
                if (code == 4) 
                    return PROVIDER_STATUS.ERROR;
    }
    
    PROVIDER_STATUS = {
        OK: {
            icon: "icon-ok",
            color: ""
        },
        WARNING: {
            icon: "icon-warning",
            color: "WARNINGcolor"
        },
        ERROR: {
            icon: "icon-error",
            color: "ERRORcolor"
        }
    }    
})();
