/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.process.authoring.web.common.ui.internal.editors.PracticeTree"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.ui.internal.editors.Toolbar"); //$NON-NLS-1$
	dojo.require("dijit.Tree"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("dijit.tree.dndSource"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.authoring.web.common.ui.internal.editors", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	var ProcessAuthoringConstants = com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants;
	var Toolbar = com.ibm.team.process.authoring.web.common.ui.internal.editors.Toolbar;
	
	dojo.declare("com.ibm.team.process.authoring.web.common.ui.internal.editors.PracticeTree", dijit.Tree, { //$NON-NLS-1$
		practiceResource: null,
		persist: false,
		isNew: false,
		viewOnly: false,
		isAssociated: false,
		isPlainTree: false,
		isOutlinePane: false,
		rootOnly: false,
		indexInParent: -1,
		treeContainer: null,
		toolBarList: null,
		addWidth: -1,
		dndController: null,
		viewMode: true,
		isSharedTree: false,
		canWriteShared: false,
		
		constructor: function(args) {
			this.toolBarList = [];
			this.dndController = "dijit.tree.dndSource"; //$NON-NLS-1$
			this.betweenThreshold = 5;
			this.viewOnly = args.isSharedTree || args.isOutlinePane || args.isAssociated;
			this.canWriteShared = args.canWriteShared;
		},
		
		postCreate: function() {
			this.inherited(arguments);
			this.dndController.singular = true;  // Only allow single drag of items
			if (this.isPlainTree || this.viewMode) {
				this.dndController.isSource = false;
			}
		},
		
		checkAcceptance: function(source, nodes) {
			// Can not drag shared practices and its children
			var practiceTree = source.tree;
			if (practiceTree.isSharedTree) {
				return false;
			}
			// Can not drag practice
			if (nodes.length > 0) {
				var treeNode = dijit.getEnclosingWidget(nodes[0]);
				var item = treeNode.item;
				if (item.root === true) {
					return false;
				}
			}
			return true;
		},
		
		checkItemAcceptance: function(target, source, position) {
			// Compute the new parent item
			this.targetAnchorNode = target;
			var targetNode = dijit.getEnclosingWidget(target);
			var targetItem = targetNode.item;
			
			this.position = position;
			// Can not drop over practice element node 
			if (targetItem.type == ProcessAuthoringConstants.NAVIGATORTYPE_ELEMENT && position === "over") { //$NON-NLS-1$
				return false;
			}
			
			var practiceTree = source.tree;
			if (practiceTree.practiceResource != targetNode.tree.practiceResource) {
				return false;
			}
			return true;
		},
		
		onDndDrop: function(source, nodes, copy) {
			if (this.containerState == "Over") { //$NON-NLS-1$
				// remember the source item (item to be moved)
				var oldTreeNode, oldItem, oldParentItem;
				if (nodes.length > 0) {
					oldTreeNode = dijit.getEnclosingWidget(nodes[0]);
					oldItem = oldTreeNode.item;
					oldParentItem = oldTreeNode.getParent().item;
				}
				var practiceTree = source.tree;
				var target = this.targetAnchorNode;
				
				// Compute the new parent item
				var targetNode = dijit.getEnclosingWidget(target);
				var newParentItem;
				if (this.position == "over") { //$NON-NLS-1$
					newParentItem = (targetNode && targetNode.item) || practiceTree.item;
				} else if (this.position == "before" || this.position == "after") { //$NON-NLS-1$  //$NON-NLS-2$
					newParentItem = (targetNode.getParent() && targetNode.getParent().item) || practiceTree.item;
				}
				
				var targetIndex = targetNode.getIndexInParent();
				if (newParentItem.root === true && targetIndex == -1) {
					targetIndex = 0;
				}
				
				if (this.position == "before" && newParentItem.id == oldParentItem.id && oldTreeNode.getIndexInParent() < targetIndex) { //$NON-NLS-1$
					targetIndex = targetIndex - 1;
				}
				
				if (this.position == "after" && !(newParentItem.id == oldParentItem.id && oldTreeNode.getIndexInParent() < targetIndex)) { //$NON-NLS-1$
					targetIndex = targetIndex + 1;
				}
				
				if (this.position == "over") { //$NON-NLS-1$
					if (newParentItem.children) {
						targetIndex = targetIndex + newParentItem.children.length;
					}
				}
				if (oldItem.type == ProcessAuthoringConstants.NAVIGATORTYPE_ELEMENT ||
					oldItem.type == ProcessAuthoringConstants.NAVIGATORTYPE_CATEGORY) {
					practiceTree.onMoveNode(oldItem, oldParentItem, newParentItem, targetIndex, practiceTree);
					practiceTree.selectNode(oldItem);
				} else if (oldItem.root === true) {
					// Currently, do not support DnD for practice
				}
			}
			this.onDndCancel();
		},
		
		onMoveNode: function(sourceItem, oldParentItem, newParentItem, position, isDnD, sourceTree) {
		
		},
		
		onShowMoveDialog: function(sourceTree, sourceItem, oldParentItem) {
		
		},
		
		_load: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.authoring.web.common.ui.internal.editors", "Messages");//$NON-NLS-1$ //$NON-NLS-2$
			this.model.getRoot(dojo.hitch(this, function(item) {
				item.isSharedElement = this.isSharedTree;
				item.id = this.practiceResource;
			}), function(err) {
			});
			this.inherited(arguments);
			this.subscribe(ProcessAuthoringConstants.PUBLISHTYPE_VIEWMODE, this._previewMode); //$NON-NLS-1$
			this.subscribe(ProcessAuthoringConstants.PUBLISHTYPE_EDITMODE, this._editMode); //$NON-NLS-1$
		},
		
		_previewMode: function() {
			this.viewMode = true;
			if (this.dndController && !dojo.isString(this.dndController)) {
				this.dndController.isSource = false;
			}
		},
		
		_editMode: function() {
			if (this.dndController && !dojo.isString(this.dndController)) {
				this.dndController.isSource = true;
			}
			this.viewMode = false;
		},
		
		onClick: function(/* dojo.data */item, /*TreeNode*/ node) {
			this.treeContainer.selectTree(this);
			if (!this.isPlainTree) {
				if (item.type == ProcessAuthoringConstants.NAVIGATORTYPE_ELEMENT) {
					this.onOpenPracticeElement(item, this.practiceResource, this.viewOnly);
				} else if (item.type == ProcessAuthoringConstants.NAVIGATORTYPE_CATEGORY) {
					this.onOpenCategory(item, this.practiceResource, this.viewOnly);
				} else if (item.root === true) {
					this.onOpenPractice(item, this.practiceResource, this.viewOnly);
				} else if (item.type == ProcessAuthoringConstants.XMLELEMENT_PRACTICE) {
					this.onOpenPractice(item, item.resource, this.viewOnly);
				}
			} else {
				//If the tree is a plain tree, do not add click event
				var moveDialog = this.treeContainer.controller;
				if (item.type == ProcessAuthoringConstants.NAVIGATORTYPE_ELEMENT) {
					if ((!moveDialog.parent.checked) && (moveDialog.sourceItem.type == ProcessAuthoringConstants.NAVIGATORTYPE_ELEMENT || moveDialog.sourceItem.type == ProcessAuthoringConstants.NAVIGATORTYPE_CATEGORY)) {
						moveDialog.okButton.disabled = false;
					} else {
						moveDialog.okButton.disabled = true;
					}
					this.treeContainer.selectedNode = node;
					this.treeContainer.selectedNodeItemIndex = node.getIndexInParent();
				} else if (item.type == ProcessAuthoringConstants.NAVIGATORTYPE_CATEGORY) {
					if (moveDialog.sourceItem.type == ProcessAuthoringConstants.NAVIGATORTYPE_CATEGORY || moveDialog.sourceItem.type == ProcessAuthoringConstants.NAVIGATORTYPE_ELEMENT) {
						moveDialog.okButton.disabled = false;
					} else {
						moveDialog.okButton.disabled = true;
					}
					this.treeContainer.selectedNode = node;
					this.treeContainer.selectedNodeItemIndex = node.getIndexInParent();
				} else if (item.root === true) {
					if (moveDialog.sourceItem.root === true || moveDialog.sourceItem.root !== true && moveDialog.parent.checked) {
						moveDialog.okButton.disabled = false;
					} else {
						moveDialog.okButton.disabled = true;
					}
					this.treeContainer.selectedNode = node;
					this.treeContainer.selectedNodeItemIndex = dojo.indexOf(this.treeContainer.trees, this);
				}
			}
		},
		
		getIconClass: function(item, opened) {
			var sharedPractice = false;
			if (item && !item.root) {
				sharedPractice = Boolean(item._S.getValue(item, "shared")); //$NON-NLS-1$
			}
			
			if (!item || item.root || item.type == ProcessAuthoringConstants.NAVIGATORTYPE_PRACTICE) {
				return (this.isSharedTree || this.editPractice || sharedPractice) ? "sharedPracticeIcon" : "privatePracticeIcon"; //$NON-NLS-1$ //$NON-NLS-2$
			} else if (!item || item.type == ProcessAuthoringConstants.NAVIGATORTYPE_CATEGORY) {
				return opened ? "categoryOpenIcon" : "categoryClosedIcon"; //$NON-NLS-1$ //$NON-NLS-2$
			} else {
				return "practiceElementIcon"; //$NON-NLS-1$
			}
		},
		
		pageX: function(elem) {
			if (dojo.style(elem, "visibility") === "hidden") { //$NON-NLS-1$ //$NON-NLS-2$
				return this.pageX(elem.previousSibling);
			} else {
				return elem.offsetParent ? elem.offsetLeft + this.pageX(elem.offsetParent) : elem.offsetLeft;
			}
		},
		
		pageRight: function(elem) {
		
			if (!elem) {
				return 0;
			}
			if (dojo.style(elem, "visibility") === "hidden") { //$NON-NLS-1$ //$NON-NLS-2$
				return this.pageRight(elem.previousSibling);
			} else {
				return this.pageX(elem) + elem.offsetWidth;
			}
		},
		
		/**
		 * Show/Hide the actions when the editor is in view mode.
		 * 
		 * @param {Widget} editToolBar - The ToolBar that contains all the actions 
		 * 
		 */
		handleButtonShowup: function(editToolBar) {
			dojo.forEach(editToolBar.getChildren(), dojo.hitch(this, function(btn) {
				if (btn.hideInViewMode) {
					dojo.toggleClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE_OCCUPY, this.viewMode);
				}
			}));
		},
		
		/**
		 * Show the actions when mouse hovering on and adjust the width of practice tree pane if needed.
		 * 
		 * @param {Widget} node - The tree node that the mouse hovers on 
		 * 
		 */
		_onNodeMouseEnter: function(/*Widget*/node) {
			if (node.editToolBar) {
				this.addWidth = -1;
				dojo.toggleClass(node.editToolBar.domNode, ProcessAuthoringConstants.CSS_HIDE_OCCUPY, false);
				this.handleButtonShowup(node.editToolBar);
				if (node.editToolBar.domNode.lastChild) {
					var nodeRight = this.pageRight(node.editToolBar.domNode.lastChild);
					var treePanelRight = this.pageRight(this.treeContainer.domNode);
					if (treePanelRight < nodeRight) {
						this.addWidth = nodeRight - treePanelRight;
						dojo.publish(ProcessAuthoringConstants.PUBLISHTYPE_RESIZE_NAVIGATOR, [this.addWidth]);
					}
				}
			}
		},
		
		/**
		 * Hide the actions after mouse leaving and adjust the width of practice tree pane if needed.
		 * 
		 * @param {Widget} node - The tree node that the mouse leaves from 
		 * 
		 */
		_onNodeMouseLeave: function(/*Widget*/node) {
		
			if (node.editToolBar) {
				dojo.toggleClass(node.editToolBar.domNode, ProcessAuthoringConstants.CSS_HIDE_OCCUPY, true);
				if (this.addWidth !== -1) {
					dojo.publish(ProcessAuthoringConstants.PUBLISHTYPE_RESIZE_NAVIGATOR, [0 - this.addWidth]);
				}
			}
		},
		
		/**
		 * Create a tree node with several actions. 
		 * Actions examples: copy URL, export practice, delete practice/element/category, etc.
		 * 
		 * @param {Object} args  The parameters needed by a tree node. For example: label, tooltip, etc.
		 * @return the newly created tree node
		 */
		_createTreeNode: function(/*Object*/args) {
			var item = args.item;
			var tnode = new dijit._TreeNode(args);
			var editToolBar = new Toolbar();
			
			var _self = this;
			
			if (this.isOutlinePane) {
				return tnode;
			}
			
			if (!item.root && this.isSharedTree) {
				item._S.setValue(item, "isSharedElement", true); //$NON-NLS-1$
			}
			
			//If the tree is a plain tree, do not add action button
            
            if (this.isPlainTree) {
            	if (this.rootOnly) {
            		if (item.root === true) {
            			return tnode;
            		} else {
            			return null;
            		}
            	} else {
            		return tnode;
            	}
            }
            
            if (item.type == ProcessAuthoringConstants.NAVIGATORTYPE_ELEMENT) {
            	//practice element
            	//copy resource url - both for editting and reading
            	if (item._S.getValue(item, "resource")) { //$NON-NLS-1$
            		editToolBar.addChild(new jazz.ui.toolbar.Button({
            			imgSrc: "com.ibm.team.process.authoring.web.common/graphics/icons/copyurl-button.gif", //$NON-NLS-1$ $
            			title: this.messages.PracticeTree_9,
            			onClick: dojo.hitch(this, function(evt) {
            				dojo.stopEvent(evt);
            				_self.onCopyResourceUrl(item._S.getValue(item, "resource")); //$NON-NLS-1$
            			})
            		}));
            	}
            	
            	//open practice
            	if (this.viewOnly) {
            		editToolBar.addChild(new jazz.ui.toolbar.Button({
            			imgSrc: "com.ibm.team.process.authoring.web.common/graphics/icons/navigate-button.gif", //$NON-NLS-1$ 
            			title: this.messages.PracticeTree_6,
            			onClick: dojo.hitch(this, function(evt) {
            				dojo.stopEvent(evt);
            				this.onNavigateSharedPractice(_self.practiceResource);
            			})
            		}));
            	} else {
            		// add move button - for editting
            		editToolBar.addChild(new jazz.ui.toolbar.Button({
            			hideInViewMode: true,
            			imgSrc: "com.ibm.team.process.authoring.web.common/graphics/icons/move-button.gif", //$NON-NLS-1$ 
            			title: this.messages.PracticeTree_14,
            			onClick: dojo.hitch(this, function(evt) {
            				dojo.stopEvent(evt);
            				_self.onShowMoveDialog(_self, item, tnode.getParent().item);
            			})
            		}));
            		
            		//					//delete button
            		editToolBar.addChild(new jazz.ui.toolbar.Button({
            			hideInViewMode: true,
            			imgSrc: "com.ibm.team.process.authoring.web.common/graphics/icons/delete-button.gif", //$NON-NLS-1$
            			title: this.messages.PracticeTree_0,
            			onClick: dojo.hitch(this, function(evt) {
            				dojo.stopEvent(evt);
             				this.onDeletePracticeElement(item, this.practiceResource, this.treeContainer);
            			})
            		}));
            	}
            } else if (item.type == ProcessAuthoringConstants.NAVIGATORTYPE_CATEGORY) {
            	// categories
            	
            	// add move button
            	if (!this.viewOnly) {
            		editToolBar.addChild(new jazz.ui.toolbar.Button({
            			hideInViewMode: true,
            			imgSrc: "com.ibm.team.process.authoring.web.common/graphics/icons/move-button.gif", //$NON-NLS-1$ 
            			title: this.messages.PracticeTree_13,
            			onClick: dojo.hitch(this, function(evt) {
            				dojo.stopEvent(evt);
            				_self.onShowMoveDialog(_self, item, tnode.getParent().item);
            			})
            		}));
            		
            		editToolBar.addChild(new jazz.ui.toolbar.Button({
            			hideInViewMode: true,
            			imgSrc: "com.ibm.team.process.authoring.web.common/graphics/icons/add-practice-element-co.gif", //$NON-NLS-1$ 
            			title: this.messages.PracticeTree_1,
            			onClick: dojo.hitch(this, function(evt) {
            				dojo.stopEvent(evt);
            				var newItem = this.onNewPracticeElement(item, _self.practiceResource);
							if(newItem) {
								this.selectNode(newItem);
							}
            			})
            		}));
            		
            		editToolBar.addChild(new jazz.ui.toolbar.Button({
            			hideInViewMode: true,
            			imgSrc: "com.ibm.team.process.authoring.web.common/graphics/icons/add-category_co.gif", //$NON-NLS-1$ 
            			title: this.messages.PracticeTree_2,
            			onClick: dojo.hitch(this, function(evt) {
            				dojo.stopEvent(evt);
            				var newItem = this.onNewNode(item, _self.practiceResource);
							if(newItem) {
								this.selectNode(newItem);
							}
            			})
            		}));
            		
            		editToolBar.addChild(new jazz.ui.toolbar.Button({
            			hideInViewMode: true,
            			imgSrc: "com.ibm.team.process.authoring.web.common/graphics/icons/delete-button.gif", //$NON-NLS-1$
            			title: this.messages.PracticeTree_3,
            			onClick: dojo.hitch(this, function(evt) {
            				dojo.stopEvent(evt);
            				this.onDeleteNode(item, _self.practiceResource);
            			})
            		}));
            	}
            } else if (item.root === true) {
            	// practices
            	
            	if (!this.isNew && !this.editPractice) {
            		editToolBar.addChild(new jazz.ui.toolbar.Button({
            			imgSrc: "com.ibm.team.process.authoring.web.common/graphics/icons/copyurl-button.gif", //$NON-NLS-1$ 
            			title: this.messages.PracticeTree_9,
            			onClick: dojo.hitch(this, function(evt) {
            				dojo.stopEvent(evt);
            				_self.onCopyResourceUrl(_self.practiceResource);
            			})
            		}));
            		
            		editToolBar.addChild(new jazz.ui.toolbar.Button({
            			imgSrc: "com.ibm.team.process.authoring.web.common/graphics/icons/export-practice_co.gif", //$NON-NLS-1$
            			title: this.messages.PracticeTree_10,
            			onClick: dojo.hitch(this, function(evt) {
            				dojo.stopEvent(evt);
            				_self.onExportPractice(_self.practiceResource, _self.label);
            			})
            		}));
            		
            		
            		
            		if (!this.viewOnly && this.canWriteShared) {
            			//currently, add it to both shared and private process description, 
            			//if it's required for private process description only, need to add checking logic
            			editToolBar.addChild(new jazz.ui.toolbar.Button({
            				hideInViewMode: true,
            				imgSrc: "com.ibm.team.process.authoring.web.common/graphics/icons/promote-practice_co.gif", //$NON-NLS-1$ 
            				title: this.messages.PracticeTree_11,
            				onClick: dojo.hitch(this, function(evt) {
            					dojo.stopEvent(evt);
            					_self.onPromotePractice(item, _self.practiceResource, _self.treeContainer);
            				})
            			}));
            		}
            	}
				// add navigate button
            	if (this.viewOnly) {
					editToolBar.addChild(new jazz.ui.toolbar.Button({
            			imgSrc: "com.ibm.team.process.authoring.web.common/graphics/icons/navigate-button.gif", //$NON-NLS-1$
            			title: this.messages.PracticeTree_8,
            			onClick: dojo.hitch(this, function(evt) {
            				dojo.stopEvent(evt);
            				this.onNavigateSharedPractice(_self.practiceResource);
            			})
            		}));
				}
				
            	// add move button
            	if (!this.editPractice && !this.isAssociated) {
            		editToolBar.addChild(new jazz.ui.toolbar.Button({
            			hideInViewMode: true,
            			imgSrc: "com.ibm.team.process.authoring.web.common/graphics/icons/move-button.gif", //$NON-NLS-1$ 
            			title: this.messages.PracticeTree_12,
            			onClick: dojo.hitch(this, function(evt) {
            				dojo.stopEvent(evt);
            				_self.onShowMoveDialog(_self, item, null); //$NON-NLS-1$
            			})
            		}));
            	}
            	
            	if (this.viewOnly) {
            		if (!this.isAssociated) {
            			editToolBar.addChild(new jazz.ui.toolbar.Button({
            				hideInViewMode: true,
            				imgSrc: "com.ibm.team.process.authoring.web.common/graphics/icons/delete-button.gif", //$NON-NLS-1$ 
            				title: this.messages.PracticeTree_4,
            				onClick: dojo.hitch(this, function(evt) {
            					dojo.stopEvent(evt);
            					this.onRemoveSharedPractice(item, _self.practiceResource, _self.treeContainer);
            				})
            			}));
            		}
            	} else if (!this.editPractice) {
            		editToolBar.addChild(new jazz.ui.toolbar.Button({
            			hideInViewMode: true,
            			imgSrc: "com.ibm.team.process.authoring.web.common/graphics/icons/add-category_co.gif", //$NON-NLS-1$ 
            			title: this.messages.PracticeTree_2,
            			onClick: dojo.hitch(this, function(evt) {
            				dojo.stopEvent(evt);
            				var newItem = this.onNewNode(item, _self.practiceResource);
							if(newItem) {
								this.selectNode(newItem);
							}
            			})
            		}));
            		
            		editToolBar.addChild(new jazz.ui.toolbar.Button({
            			hideInViewMode: true,
            			imgSrc: "com.ibm.team.process.authoring.web.common/graphics/icons/add-practice-element-co.gif", //$NON-NLS-1$ 
            			title: this.messages.PracticeTree_1,
            			onClick: dojo.hitch(this, function(evt) {
            				dojo.stopEvent(evt);
            				var newItem = this.onNewPracticeElement(item, _self.practiceResource);
							if(newItem) {
								this.selectNode(newItem);
							}
            			})
            		}));
            		
            		editToolBar.addChild(new jazz.ui.toolbar.Button({
            			hideInViewMode: true,
            			imgSrc: "com.ibm.team.process.authoring.web.common/graphics/icons/delete-button.gif", //$NON-NLS-1$ 
            			title: this.messages.PracticeTree_7,
            			onClick: dojo.hitch(this, function(evt) {
            				dojo.stopEvent(evt);
            				this.onDeletePractice(item, _self.practiceResource, _self.treeContainer);
            			})
            		}));
            	}
            }
			
			dojo.toggleClass(editToolBar.domNode, ProcessAuthoringConstants.CSS_HIDE_OCCUPY, true);
			tnode.editToolBar = editToolBar;
			tnode.contentNode.appendChild(editToolBar.domNode);
			this.toolBarList.push(editToolBar);
			
			return tnode;
		},
		
		selectNode: function(item) {
			var nodes = this.getNodesByItem(item);
			if(nodes.length > 0) {
				var node = nodes[0];
				this.treeContainer.selectTree(this);
				setTimeout(dojo.hitch(this, function() {
					this.set("path", node.getTreePath()); //$NON-NLS-1$
				}), 200);
			}
		},
				
		onNewPracticeElement: function(item, practiceResource) {
			return null;
		},
		
		onDeletePracticeElement: function(item, practiceResource, treeContainer) {
		
		},
		
		onNewNode: function(item, practiceResource) {
			return null;
		},
		
		onExportPractice: function(resourceUrl, practiceName) {
		
		},
		
		onCopyResourceUrl: function(resourceUrl) {
		
		},
		
		onDeleteNode: function(item, practiceResource) {
		
		},
		onDeletePractice: function(item, practiceResource, treeContainer) {
		
		},
		onRemoveSharedPractice: function(item, practiceResource, treeContainer) {
		
		},
		
		onNavigateSharedPractice: function(practiceResource) {
		
		},
		
		onEditCategoryTitle: function(item) {
		
		},
		onOpenPractice: function(item, node, practiceResource) {
		
		},
		onOpenPracticeElement: function(item, practiceResource) {
		
		},
		
		onError: function() {
		},
		
		onOpenCategory: function(practiceResource, item) {
		
		},
		
		destroy: function() {
			dojo.forEach(this.toolBarList, function(toolBar) {
				toolBar.destroyRecursive();
			});
			
			this.inherited(arguments);
		}
	});
	
})();
