/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.util.DashboardUtil");

dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("net.jazz.ajax.contextRoot");

dojo.require("jazz.app.internal.applicationAbout");

(function() {

/*
 * Aliases
 */
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;

/*
 * Constants 
 */
var QM_APP_ID = "http://jazz.net/application/qm";

/*
 * Public utility methods.
 * 
 * PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * future with minimal or no warning.
 * 
 * @since 0.7
 */
com.ibm.team.dashboard.web.util.DashboardUtil = {
	/*
	 * Returns an HREF that can be used as a link to the given dashboard info. Tab is
	 * optional, and if specified, will take you to that specific tab. The scopeItem
	 * param indicates the target dashboard depending on the indicated scope. For
	 * personal dashboards, scopeItem is the id of the dashboard, for team dashboards
	 * it's an array of segments in the team area path and for project dashboards it's
	 * the project area name.
	 * 
	 * If projectName is specified, it will be shown in the context of that project,
	 * otherwise it will be assumed to be in the current project context.
	 */
	/* string */ getDashboardHref: function(/* string */ scope, /* string */ scopeItem, /* string */ tabId, /* string */ projectName, /* string */ webURL) {
		var href = [];
		if (projectName) {
			href = [webURL || net.jazz.ajax.contextRoot()];
			if (this._getCurrentApplicationID() === QM_APP_ID) {
				// QM
				// See: 202715: [Teams Viewlet]: Not able to navigate to any QM team dashboards on IE
				href.push("/web/console/");
			} else {
				// CCM (and rest)
				href.push("/web/projects/");
			}
			href.push(encodeURIComponent(projectName));
			if (djConfig.isDebug) {
				href.push("?debug=true");
			}
		}
		switch(scope) {
			case DashboardConstants.SCOPE_CONTRIBUTOR:
				href.push(net.jazz.ajax.contextRoot());
				href.push("/dashboards/");
				href.push(scopeItem);
				break;
			case DashboardConstants.SCOPE_TEAM_AREA:
				href.push("#action=");
				href.push(DashboardConstants.ACTION_VIEW_DASHBOARD);
				var path = [];
				for (var i = 0; i < scopeItem.length; i++) {
					if (path.length > 0) {
						path.push("/");
					}
					path.push(encodeURIComponent(scopeItem[i]));
				}
				href.push("&");
				href.push(DashboardConstants.URL_PARAM_TEAM);
				href.push("=");
				href.push(path.join(""));
				break;
			case DashboardConstants.SCOPE_PROJECT_AREA:
				href.push("#action=");
				href.push(DashboardConstants.ACTION_VIEW_DASHBOARD);
				break;
		}
		if (tabId) {
			href.push("&");
			href.push(DashboardConstants.URL_PARAM_TAB);
			href.push("=");
			href.push(tabId);
		}
		return href.join("");
	},
	
	/* string */ _getCurrentApplicationID: function() {
		var about = jazz.app.internal.applicationAbout.getCurrent();
		if (about &&
				about.identifier) {
			return about.identifier;
		}
		return null;
	}	
	
};

})();